'use strict';
var self = ConfigHandler;
module.exports = self;
var _ = require('underscore');
var url = require('url');
var util = require('util');
var ConfigurationManager = require('@jfrog/configuration-manager');
var fs = require('fs');
var path = require('path');
exports = ConfigHandler;
module.exports = ConfigHandler;
exports.setLoggingConfig = setLoggingConfig;
var configManager = ConfigurationManager.default({
    productHome: '/opt/jfrog/pipelines',
    microservice: 'pipelines.www',
    aliases: {}
});
function setLoggingConfig() {
    global.config = global.config || {};
    global.config.logging = {
        useOldLogger: !!configManager.getBool(`pipelines.logging.useOldLogger`),
        service: {
            level: configManager.get(`pipelines.logging.application.level`) || 'warn',
            rotation: {
                maxSizeMb: configManager
                    .getInt(`pipelines.logging.application.rotation.maxSizeMb`) || 10,
                maxFiles: configManager
                    .getInt(`pipelines.logging.application.rotation.maxFiles`) || 10,
            },
            streamToStdout: configManager
                .getBool(`pipelines.logging.application.streamToStdout`) || false
        },
        streamToJson: configManager.getBool(`shared.logging.enableJsonConsoleLogAppenders`) || false
    };
}
function ConfigHandler() {
    this.globalConfig = {};
    this.globalConfig.runMode = configManager.get('core.runMode');
    this.globalConfig.controlplaneVersion =
        configManager.get('core.controlplaneVersion');
    this.globalConfig.buildplaneVersion = configManager.get('core.buildplaneVersion');
    this.globalConfig.isSaas = configManager.get('runtime.isSaas') || false;
    this.globalConfig.ignoreTLSErrors = configManager.get('core.ignoreTLSErrors') ||
        false;
    this.globalConfig.installUser = configManager.get('core.user') || 0;
    this.globalConfig.apiToken =
        configManager.get('core.installerAuthToken');
    var dbType = configManager.get('shared.db.type');
    var dbIp = configManager.get('shared.db.ip');
    var dbPort = configManager.get('shared.db.port');
    var dbName = configManager.get('shared.db.name');
    var dbUsername = configManager.get('shared.db.username');
    var dbPassword = configManager.get('shared.db.password');
    var dbHost;
    if (dbIp && dbPort)
        dbHost = util.format('%s:%s', dbIp, dbPort);
    else
        dbHost = dbIp;
    if (!_.isUndefined(dbType) && !_.isUndefined(dbHost) &&
        !_.isUndefined(dbName) && !_.isUndefined(dbUsername) &&
        !_.isUndefined(dbPassword))
        this.globalConfig.dbConnectionString = util.format('%s://%s:%s@%s/%s', dbType, dbUsername, dbPassword, dbHost, dbName);
    var rtUIUrl = configManager.get('shared.artifactory.baseUrlUI') || '';
    this.globalConfig.rtUIUrl = '';
    if (rtUIUrl) {
        this.globalConfig.rtUIUrl = rtUIUrl.replace(/\/+$/, '');
    }
    this.globalConfig.rtServiceId =
        configManager.get('shared.artifactory.serviceId');
    var splitServiceId = this.globalConfig.rtServiceId.split('@');
    this.globalConfig.wwwServiceId = splitServiceId[0] + 'www@' +
        (splitServiceId[1] || '');
    this.globalConfig.rtLicenseId =
        configManager.get('shared.artifactory.licenseId');
    this.globalConfig.artifactoryUrl =
        util.format('http://127.0.0.1:%s/artifactory', this.globalConfig.rtRouterAccessPort);
    var rtRouterJoinKey = configManager.get('shared.router.joinKey');
    if (_.isEmpty(rtRouterJoinKey)) {
        var joinKeyPath = path.join('/opt/jfrog/pipelines/var/etc/security', 'join.key');
        if (fs.existsSync(joinKeyPath)) {
            try {
                var encryptedJoinKey = fs.readFileSync(joinKeyPath, 'utf8').trim();
                try {
                    rtRouterJoinKey = configManager.getDecryptedValue(encryptedJoinKey).trim();
                }
                catch (err) {
                    logger.error('Failed to decrypt joinkey with error : ' + err);
                }
            }
            catch (err) {
                logger.error('Failed to read joinKey from file ' + joinKeyPath +
                    'with error: ' + err);
            }
        }
    }
    this.globalConfig.rtRouterJoinKey = rtRouterJoinKey;
    this.globalConfig.rtRouterAccessPort =
        configManager.get('shared.router.accessPort');
    this.globalConfig.rtRouterDataPort =
        configManager.get('shared.router.dataPort');
    this.globalConfig.amqpExchange = configManager.get('shared.msg.defaultExchange');
    this.globalConfig.rootQueueList = configManager.get('shared.msg.queues');
    var rootVhost = configManager.get('shared.msg.amqpRootVhost') || 'pipelinesRoot';
    var msgIP = configManager.get('shared.msg.ip');
    var msgPort = configManager.get('shared.msg.port');
    var msgUsername = configManager.get('shared.msg.username');
    var msgPassword = configManager.get('shared.msg.password');
    var msgProtocol = configManager.get('shared.msg.protocol') || 'amqp';
    if (!_.isUndefined(msgUsername) && !_.isUndefined(msgPassword) &&
        !_.isUndefined(msgIP) && !_.isUndefined(msgPort) &&
        !_.isUndefined(rootVhost))
        this.globalConfig.rootAmqpUrl = util.format('%s://%s:%s@%s:%s/%s', msgProtocol, msgUsername, msgPassword, msgIP, msgPort, rootVhost);
    var redisIP = configManager.get('shared.redis.ip');
    var redisPort = configManager.get('shared.redis.port');
    var redisPassword = configManager.get('shared.redis.password');
    var sentinelPort = configManager.get('shared.sentinel.port');
    var sentinelGroup = configManager.get('shared.sentinel.masterGroupName');
    this.globalConfig.sentinelPort = sentinelPort;
    this.globalConfig.sentinelGroup = sentinelGroup;
    if (redisIP && redisPort) {
        if (!_.isUndefined(redisPassword)) {
            this.globalConfig.redisUrl = util.format('%s:%s:%s', redisIP, redisPort, redisPassword);
        }
        else {
            this.globalConfig.redisUrl = util.format('%s:%s', redisIP, redisPort);
        }
    }
    this.globalConfig.redisClusterEnabled =
        configManager.get('shared.redis.clusterEnabled');
    if (_.isUndefined(this.globalConfig.redisClusterEnabled))
        this.globalConfig.redisClusterEnabled = true;
    this.globalConfig.apiUrl =
        util.format('http://127.0.0.1:%s/pipelines/api/v1', this.globalConfig.rtRouterAccessPort);
    this.globalConfig.customCertsEnabled = false;
    var customCertsPath = configManager.get('core.customCACertsPath');
    if (!_.isEmpty(customCertsPath)) {
        this.globalConfig.customCertsPath = customCertsPath;
        this.globalConfig.customCertsEnabled = true;
    }
    this.globalConfig.internalApiUrl = this.globalConfig.apiUrl;
    this.globalConfig.publicApiUrl =
        configManager.get('core.services.api.externalUrl');
    if (_.isUndefined(this.globalConfig.publicApiUrl))
        this.globalConfig.publicApiUrl = this.globalConfig.apiUrl;
    else
        this.globalConfig.publicApiUrl = this.globalConfig.publicApiUrl + '/v1';
    var wwwPort = configManager.get('core.services.www.port');
    this.globalConfig.wwwPort = wwwPort;
    this.globalConfig.wwwUrl = '';
    if (this.globalConfig.rtUiUrl) {
        this.globalConfig.wwwUrl = util.format('%s/pipelines/www', this.globalConfig.rtUiUrl);
    }
    this.globalConfig.sessionSecret =
        configManager.get('core.services.www.sessionSecret');
    this.globalConfig.defaultMinionCount =
        configManager.get('runtime.defaultMinionCount') || 0;
    this.globalConfig.stepTimeoutMS = configManager.get('runtime.stepTimeoutMS');
    this.globalConfig.allowDynamicNodes =
        configManager.get('runtime.allowDynamicNodes') || false;
    this.globalConfig.allowSystemNodes =
        configManager.get('runtime.allowSystemNodes') || false;
    this.globalConfig.allowCustomNodes =
        configManager.get('runtime.allowCustomNodes') || true;
    this.globalConfig.customNodesAdminOnly =
        configManager.get('runtime.customNodesAdminOnly') || false;
    this.globalConfig.defaultClusterType =
        configManager.get('runtime.defaultClusterType');
    this.globalConfig.maxDiskUsagePercentage =
        configManager.get('runtime.maxDiskUsagePercentage');
    this.globalConfig.nodeCacheIntervalMS =
        configManager.get('runtime.nodeCacheIntervalMS');
    this.globalConfig.nodeStopDayOfWeek =
        configManager.get('runtime.nodeStopDayOfWeek') || 0;
    this.globalConfig.nodeStopIntervalDays =
        configManager.get('runtime.nodeStopIntervalDays');
    this.globalConfig.rootBucket = configManager.get('runtime.rootBucket');
    this.globalConfig.maxClusterNodes =
        configManager.get('runtime.maxClusterNodes') || 3;
    this.globalConfig.hardDeleteIntervalInMins =
        configManager.get('core.hardDeleteIntervalInMins');
    this.globalConfig.maxNodeCheckInDelayInMinutes =
        configManager.get('runtime.maxNodeCheckInDelayMin');
    this.globalConfig.nodeId = process.env.PIPELINES_NODE_ID ||
        configManager.get('core.id');
    this.globalConfig.pipelinesNodeId = this.globalConfig.nodeId;
    this.globalConfig.installerArchitecture =
        configManager.get('core.architecture');
    this.globalConfig.installerOS = configManager.get('core.os');
    this.globalConfig.installerOSDistribution =
        configManager.get('core.osDistribution');
    this.globalConfig.allowCallHome = configManager.get('core.allowCallHome') || false;
    this.globalConfig.callHomeUrl = configManager.get('core.callHomeUrl');
    this.globalConfig.serviceInstanceHealthCheckIntervalInMins =
        configManager.get('core.serviceInstanceHealthCheckIntervalInMins') || 1;
    this.globalConfig.serviceInstanceStatsCutOffIntervalInHours =
        configManager.get('core.serviceInstanceStatsCutOffIntervalInHours') || 24;
    this.globalConfig.apiRetryInterval = 3;
    const servicesObject = configManager.get('core.services');
    this.globalConfig.servicesName = Object.keys(servicesObject).map(val => val.toLocaleLowerCase());
    this.globalConfig.runOnePodInOneNode = configManager.get('runtime.runOnePodInOneNode') || true;
}
//# sourceMappingURL=ConfigHandler.js.map