'use strict';
var async = require('async');
var util = require('util');
var parseJsonBody = require('./parseJsonBody');
var self = PipelinesAdapter;
module.exports = self;
var baseUrl;
const SERVICE_TYPE = 'jfpwww';
function PipelinesAdapter(token, type) {
    logger.trace('Initializing', self.name);
    baseUrl = config.internalApiUrl;
    this.headers = {
        'Content-Type': 'application/json; charset=utf-8',
        'user-agent': `JFrog-${SERVICE_TYPE}`
    };
    if (token) {
        if (type === 'bearer') {
            this.headers.Authorization = `Bearer ${token}`;
        }
        else {
            this.headers.Authorization = `apiToken ${token}`;
        }
    }
    if (token)
        this.token = token;
    this.type = type;
}
PipelinesAdapter.prototype.get = function (relativeUrl, callback) {
    logger.trace('Pipelines GET data ', relativeUrl);
    var opts = {
        method: 'GET',
        url: relativeUrl.indexOf('http') === 0 ?
            relativeUrl : baseUrl + relativeUrl,
        headers: this.headers,
        timeout: 180000
    };
    var bag = {
        opts: opts,
        relativeUrl: relativeUrl,
        token: this.token
    };
    async.series([
        performCall.bind(null, bag),
        parseResponse.bind(null, bag)
    ], function () {
        callback(bag.err, bag.parsedBody, bag.res);
    });
};
PipelinesAdapter.prototype.post = function (relativeUrl, json, callback) {
    logger.trace('Pipelines POST data ', relativeUrl);
    var opts = {
        method: 'POST',
        url: relativeUrl.indexOf('http') === 0 ?
            relativeUrl : baseUrl + relativeUrl,
        headers: this.headers,
        json: json,
        timeout: 180000
    };
    var bag = {
        opts: opts,
        relativeUrl: relativeUrl,
        token: this.token
    };
    async.series([
        performCall.bind(null, bag),
        parseResponse.bind(null, bag)
    ], function () {
        callback(bag.err, bag.parsedBody, bag.res);
    });
};
PipelinesAdapter.prototype.put = function (relativeUrl, json, callback) {
    logger.trace('Pipelines PUT data ', relativeUrl);
    var opts = {
        method: 'PUT',
        url: relativeUrl.indexOf('http') === 0 ?
            relativeUrl : baseUrl + relativeUrl,
        headers: this.headers,
        json: json,
        timeout: 180000
    };
    var bag = {
        opts: opts,
        relativeUrl: relativeUrl,
        token: this.token
    };
    async.series([
        performCall.bind(null, bag),
        parseResponse.bind(null, bag)
    ], function () {
        callback(bag.err, bag.parsedBody, bag.res);
    });
};
PipelinesAdapter.prototype.delete = function (relativeUrl, callback) {
    logger.trace('Pipelines DELETE data ', relativeUrl);
    var opts = {
        method: 'DELETE',
        url: relativeUrl.indexOf('http') === 0 ? relativeUrl :
            baseUrl + relativeUrl,
        headers: this.headers,
        timeout: 180000
    };
    var bag = {
        opts: opts,
        relativeUrl: relativeUrl,
        token: this.token
    };
    async.series([
        performCall.bind(null, bag),
        parseResponse.bind(null, bag)
    ], function () {
        callback(bag.err, bag.parsedBody, bag.res);
    });
};
function performCall(bag, next) {
    var who = self.name + '|' + performCall.name;
    logger.trace(who, 'Inside');
    bag.startedAt = Date.now();
    request(bag.opts, function (err, res, body) {
        var interval = Date.now() - bag.startedAt;
        logger.trace(who, 'Pipelines request ' + bag.opts.method + ' ' +
            bag.relativeUrl + ' took ' + interval +
            ' ms and returned HTTP status ' + (res && res.statusCode));
        bag.res = res;
        bag.body = body;
        if (res && res.statusCode > 299)
            err = err || res.statusCode;
        if (err) {
            logger.error(`${who} Pipelines returned status ${err} and ` +
                `message "${bag.body && JSON.stringify(bag.body)}" for request ${bag.relativeUrl}`);
            bag.err = err;
        }
        next();
    });
}
function parseResponse(bag, next) {
    var who = self.name + '|' + parseResponse.name;
    logger.trace(who, 'Inside');
    if (bag.body) {
        if (typeof bag.body === 'object') {
            bag.parsedBody = bag.body;
        }
        else {
            bag.parsedBody = parseJsonBody(bag);
        }
    }
    next();
}
PipelinesAdapter.prototype.getProjectById = function (id, callback) {
    var url = '/projects/' + id;
    this.get(url, callback);
};
PipelinesAdapter.prototype.getProjects = function (query, callback) {
    var url = '/projects?' + query;
    this.get(url, callback);
};
PipelinesAdapter.prototype.getSubscriptionAccounts =
    function (query, callback) {
        var url = '/subscriptionAccounts?' + query;
        this.get(url, callback);
    };
PipelinesAdapter.prototype.getProviders = function (query, callback) {
    var url = '/Providers?' + query;
    this.get(url, callback);
};
PipelinesAdapter.prototype.getIdentities = function (callback) {
    var url = '/identities';
    this.get(url, callback);
};
PipelinesAdapter.prototype.getSystemServices = function (query, callback) {
    var url = util.format('/systemServices?%s', query);
    this.get(url, callback);
};
PipelinesAdapter.prototype.getSystemServiceInstances =
    function (query, callback) {
        var url = util.format('/systemServiceInstances?%s', query);
        this.get(url, callback);
    };
PipelinesAdapter.prototype.getSystemCodes = function (query, callback) {
    var url = util.format('/systemCodes?%s', query);
    this.get(url, callback);
};
PipelinesAdapter.prototype.getSystemConfigByName = function (configName, callback) {
    var url = '/systemConfigs/' + configName;
    this.get(url, callback);
};
PipelinesAdapter.prototype.getIdentityTokens = function (query, callback) {
    var url = util.format('/identityTokens?%s', query);
    this.get(url, callback);
};
PipelinesAdapter.prototype.postSystemServiceInstance =
    function (body, callback) {
        var url = '/systemServiceInstances';
        this.post(url, body, callback);
    };
PipelinesAdapter.prototype.putSystemServiceInstanceById =
    function (id, body, callback) {
        var url = '/systemServiceInstances/' + id;
        this.put(url, body, callback);
    };
//# sourceMappingURL=PipelinesAdapter.js.map