'use strict';
const { join } = require('path');
var winston = require('winston');
var WinstonFileTransport = winston.transports.File;
var WinstonConsoleTransport = winston.transports.Console;
const { LOGGING, WWW_SERVICE_TYPE, WWW_SERVICE_NAME } = require('./constants');
const { LoggerManagerBuilder, AppenderBuilder, AppenderType, ThreadLocal } = require('@jfrog/nodejs-commons');
require('./ConfigHandler').setLoggingConfig();
const logging = config.logging;
configLevel();
const serviceAppender = new AppenderBuilder(logging.streamToJson ?
    AppenderType.SERVICE_SAAS : AppenderType.SERVICE)
    .setSuppliers({ traceId: () => ThreadLocal.traceId })
    .setRotation(logging.service.rotation)
    .setLevel(logging.service.level)
    .build();
const loggerManager = new LoggerManagerBuilder()
    .addAppender(serviceAppender)
    .setFilenameNesting(0)
    .setLogDirectory(LOGGING.DIR)
    .setRootDir(join(__dirname, '..', '..'))
    .setServiceColor(LOGGING.SERVICE_COLOR)
    .setWinstonRotation(true)
    .setServiceName(WWW_SERVICE_NAME)
    .setServiceType(WWW_SERVICE_TYPE)
    .build();
exports = winston;
module.exports = winston;
global.logger = getLogger();
exports.configLevel = configLevel;
exports.changeOldLoggerLevel = changeOldLoggerLevel;
exports.changeNewLoggerServiceLevel = changeNewLoggerServiceLevel;
function changeOldLoggerLevel(level) {
    winston.level = level;
}
function getLogger() {
    return logging.useOldLogger ?
        winston : loggerManager.getServiceLogger(undefined, logging.streamToJson ? { streamToJson: true } : { streamToJson: false });
}
function changeNewLoggerServiceLevel(level) {
    if (!level) {
        return;
    }
    if (logging.useOldLogger) {
        changeOldLoggerLevel(level);
        return;
    }
    loggerManager.changeAppenderLevel(logging.streamToJson ? AppenderType.SERVICE_SAAS : AppenderType.SERVICE, level);
}
function configLevel(config) {
    var jFrogLogLevels = {
        levels: {
            error: 0,
            warn: 1,
            info: 2,
            debug: 3,
            trace: 4
        },
        colors: {
            error: 'red',
            warn: 'yellow',
            info: 'green',
            debug: 'cyan',
            trace: 'blue'
        }
    };
    winston.clear();
    winston.setLevels(jFrogLogLevels.levels);
    winston.addColors(jFrogLogLevels.colors);
    config = config || {};
    winston.add(WinstonConsoleTransport, {
        timestamp: true,
        colorize: true,
        level: config.logLevel || 'info',
        json: false
    });
    winston.add(WinstonFileTransport, {
        name: 'file#out',
        timestamp: true,
        colorize: true,
        filename: join(LOGGING.DIR, 'www.log'),
        maxsize: 10 * 1024 * 1024,
        maxFiles: 20,
        level: config.logLevel,
        json: true,
        tailable: true
    });
}
//# sourceMappingURL=logger.js.map