"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const PipelinesAdapter = require('./PipelinesAdapter.js');
const configuration_1 = require("@jfrog-pipelines/configuration");
const { changeNewLoggerServiceLevel } = require('./logger.js');
const { WWW_SERVICE_NAME } = require('./constants');
const LOGGER_CONFIG_KEY_NAME = 'loggers';
const TIMEOUT_IN_MILLISECONDS = configuration_1.ConfigFactory.config.systemConfigRefreshIntervalInSeconds * 1000;
async function run() {
    const who = `micro|_global|loggerConfigJob.js|${run.name}|${WWW_SERVICE_NAME}`;
    logger.trace(who, 'Starting');
    try {
        await execute(WWW_SERVICE_NAME);
        logger.debug(`Successfully logger run execute for: ${who}`);
    }
    catch (err) {
        logger.warn(who, `Failed to set logging level with error: ${err.message}`);
    }
    finally {
        logger.debug(who, `Sleeping for ${TIMEOUT_IN_MILLISECONDS} ms before fetching log levels`);
        setTimeout(() => run(), TIMEOUT_IN_MILLISECONDS);
    }
}
async function execute(serviceName) {
    const lowerCaseServiceName = serviceName.toLowerCase();
    const who = `micro|_global|loggerConfigJob.js|${execute.name}|${lowerCaseServiceName}`;
    logger.trace(who, 'Starting');
    const MS_SERVICE_NAMES = global.config.servicesName;
    if (!MS_SERVICE_NAMES.includes(lowerCaseServiceName)) {
        throw new Error(`invalid service name "${lowerCaseServiceName}" supplied for changing log level`);
    }
    const internalSuperUserToken = global.config.apiToken;
    const suAdapter = new PipelinesAdapter(internalSuperUserToken);
    return new Promise((resolve, reject) => {
        suAdapter.getSystemConfigByName(LOGGER_CONFIG_KEY_NAME, (err, result) => {
            if (err) {
                logger.warn(who, 'Failed get system config.', err);
                reject(err);
            }
            const serviceLevel = result?.configValue?.[lowerCaseServiceName];
            if (serviceLevel) {
                logger.debug(`Might change log level for "${lowerCaseServiceName}" ${JSON.stringify(serviceLevel)}`);
                if (serviceLevel.configuredLevel === null) {
                    changeNewLoggerServiceLevel(serviceLevel.defaultLevel);
                }
                else if (serviceLevel.configuredLevel !== serviceLevel.defaultLevel) {
                    changeNewLoggerServiceLevel(serviceLevel.configuredLevel);
                }
            }
            else {
                logger.debug(`No matching systemConfig found service "${lowerCaseServiceName}" no changes to log level`);
            }
            resolve();
        });
    });
}
module.exports = run;
//# sourceMappingURL=loggerConfigJob.js.map