'use strict';
var async = require('async');
var util = require('util');
var amqplib = require('amqplib/callback_api');
var healthCheck = require('./monitors/healthCheck.js');
var routes = require('./Routes.js');
var path = require('path');
var fs = require('fs');
var uuid = require('node-uuid');
var _ = require('underscore');
var PipelinesAdapter = require('../common/PipelinesAdapter.js');
const { isUndefined } = require('lodash');
var self = MicroService;
module.exports = self;
var SOCKET_STATS_LOG_INTERVAL = 1000 * 60 * 60;
var numConnections = 0;
var numDisconnections = 0;
var numJoins = 0;
var numLeaves = 0;
var numAccRoomJoins = 0;
var numRunJobRoomJoins = 0;
var numJobConsoleRoomJoins = 0;
var subRoomStats = {};
setInterval(function () {
    logger.warn('SOCKETS HOURLY REPORT' +
        '\nNo. of connections opened: ' + numConnections +
        '\nNo. of connections closed: ' + numDisconnections +
        '\nNo. of joins: ' + numJoins +
        '\nNo. of leaves: ' + numLeaves +
        '\nNo. of acc room joins: ' + numAccRoomJoins +
        '\nNo. of run job room joins: ' + numRunJobRoomJoins +
        '\nNo. of job console room joins: ' + numJobConsoleRoomJoins +
        '\nSub room joins by account: ' + JSON.stringify(subRoomStats));
    numConnections = 0;
    numJoins = 0;
    numLeaves = 0;
    numDisconnections = 0;
    numAccRoomJoins = 0;
    numRunJobRoomJoins = 0;
    numJobConsoleRoomJoins = 0;
    subRoomStats = {};
}, SOCKET_STATS_LOG_INTERVAL);
function MicroService(callback) {
    logger.info('Starting MicroService', process.title);
    this.AMQPConnection = {
        connection: null,
        channel: null,
        closing: false,
        blocked: false,
    };
    this.isRouteInitSuccess = false;
    this.timeoutLength = 1;
    this.timeoutLimit = 180;
    return this.init(callback);
}
MicroService.prototype.init = function (callback) {
    logger.info(process.title + ' is initializing');
    async.series([
        this.registerSystemServiceInstance.bind(this),
        healthCheck,
        this.establishRabbitMQConnection.bind(this),
        this.connectToRedis.bind(this),
        this.initializeRoutes.bind(this)
    ], function (err) {
        require('../common/loggerConfigJob.js')();
        return callback(err);
    }.bind(this));
};
MicroService.prototype.registerSystemServiceInstance = function (next) {
    var who = process.title + '|registerSystemServiceInstance';
    logger.info(who);
    var bag = {
        who: who,
        suAdapter: new PipelinesAdapter(global.config.apiToken),
        createSystemServiceInstance: true
    };
    async.series([
        __getSystemServices.bind(null, bag),
        __getSystemServiceInstances.bind(null, bag),
        __generateInstanceId.bind(null, bag),
        __postSystemServiceInstace.bind(null, bag),
        __updateSystemServiceInstance.bind(null, bag)
    ], function (err) {
        if (!err)
            global.systemServiceInstanceId = bag.systemServiceInstance.id;
        return next(err);
    });
};
function __getSystemServices(bag, next) {
    var who = bag.who + '|' + __getSystemServices.name;
    logger.debug(who);
    bag.suAdapter.getSystemServices('names=www', function (err, systemServices) {
        if (err) {
            logger.error('Failed to getSystemServices for names=www with error: ' +
                err.message);
            return next(err);
        }
        if (_.isEmpty(systemServices)) {
            logger.error('getSystemServices for names=www' +
                'is empty');
            return next(err);
        }
        bag.systemService = systemServices[0];
        return next();
    });
}
function __getSystemServiceInstances(bag, next) {
    var who = bag.who + '|' + __getSystemServiceInstances.name;
    logger.debug(who);
    var query = util.format('systemServiceIds=%s&pipelineNodeIds=%s', bag.systemService.id, global.pipelinesNodeId);
    bag.suAdapter.getSystemServiceInstances(query, function (err, systemServiceInstances) {
        if (err) {
            logger.error('Failed to getSystemServiceInstances for query: ' +
                query + ' with error: ' + err.message);
            return next(err);
        }
        if (!_.isEmpty(systemServiceInstances)) {
            bag.createSystemServiceInstance = false;
            bag.systemServiceInstance = systemServiceInstances[0];
        }
        return next();
    });
}
function __generateInstanceId(bag, next) {
    if (!bag.createSystemServiceInstance)
        return next();
    var who = bag.who + '|' + __generateInstanceId.name;
    logger.trace(who, 'Inside');
    var innerBag = {
        who: who,
        instanceId: null,
        file: path.join('/opt/jfrog/pipelines/var/tmp', 'instance_id')
    };
    if (process.env.ONEBOX === 'true') {
        innerBag.file = path.join('/opt/jfrog/pipelines/var/tmp', process.env.COMPONENT + '_instance_id');
    }
    async.series([
        ___checkIfInstanceIdFileExists.bind(null, innerBag),
        ___generateAndSaveInstanceId.bind(null, innerBag)
    ], function (err) {
        if (err) {
            logger.error('Could not create instance id: ' +
                util.inspect(err));
            return next(err);
        }
        bag.instanceId = innerBag.instanceId;
        return next();
    });
}
function ___checkIfInstanceIdFileExists(bag, next) {
    var who = bag.who + '|' + ___checkIfInstanceIdFileExists.name;
    logger.trace(who, 'Inside');
    fs.access(bag.file, fs.constants.F_OK, function (err) {
        if (!err)
            bag.instanceId = fs.readFileSync(bag.file).toString();
        return next();
    });
}
function ___generateAndSaveInstanceId(bag, next) {
    if (bag.instanceId)
        return next();
    var who = bag.who + '|' + ___generateAndSaveInstanceId.name;
    logger.trace(who, 'Inside');
    bag.instanceId = uuid.v4();
    fs.writeFileSync(bag.file, bag.instanceId, 'utf8');
    return next();
}
function __postSystemServiceInstace(bag, next) {
    if (!bag.createSystemServiceInstance)
        return next();
    var who = bag.who + '|' + __postSystemServiceInstace.name;
    logger.debug(who, 'Inside');
    var body = {
        systemServiceId: bag.systemService.id,
        pipelinesNodeId: global.config.pipelinesNodeId,
        instanceId: bag.instanceId,
        statusCode: bag.systemService.desiredStatusCode
    };
    bag.suAdapter.postSystemServiceInstance(body, function (err, systemServiceInstance) {
        if (err) {
            logger.error('Failed to post systemServiceInstance with error: ' +
                err.message);
            return next(err);
        }
        bag.systemServiceInstance = systemServiceInstance;
        return next();
    });
}
function __updateSystemServiceInstance(bag, next) {
    if (bag.createSystemServiceInstance)
        return next();
    var who = bag.who + '|' + __updateSystemServiceInstance.name;
    logger.debug(who, 'Inside');
    var update = {
        statusCode: bag.systemService.desiredStatusCode
    };
    bag.suAdapter.putSystemServiceInstanceById(bag.systemServiceInstance.id, update, function (err, systemServiceInstance) {
        if (err) {
            logger.error('Failed to put systemServiceInstance: ' +
                bag.systemServiceInstance.id + ' with error: ' + err.message);
            return next(err);
        }
        bag.systemServiceInstance = systemServiceInstance;
        return next();
    });
}
MicroService.prototype.establishRabbitMQConnection = function (next) {
    var who = process.title + '|establishRabbitMQConnection';
    logger.info(who, 'is connecting to rabbitMQ');
    async.series([
        this.createAmqpConnection.bind(this),
        this.connectChannel.bind(this),
        this.connectToQueue.bind(this)
    ], function (err) {
        if (err && !next && !this.AMQPConnection.connection)
            return this.rabbitMQError(err);
        if (next)
            return next(err);
    }.bind(this));
};
MicroService.prototype.createAmqpConnection = function (next) {
    var who = process.title + '|createAmqpConnection';
    logger.info(who, 'is connecting to RabbitMQ');
    var connectionUrl = config.amqpUrl + '?heartbeat=30';
    var callbackCalled = false;
    var options = {
        rejectUnauthorized: false
    };
    amqplib.connect(connectionUrl, options, function (err, connection) {
        if (callbackCalled)
            return;
        callbackCalled = true;
        if (err)
            return next(err);
        this.AMQPConnection.connection = connection;
        this.AMQPConnection.closing = false;
        this.AMQPConnection.blocked = false;
        this.AMQPConnection.connection.on('blocked', function () {
            logger.debug(util.format('Connection blocked %s to queue', process.title));
            this.AMQPConnection.blocked = true;
            this.pollBlockedStatus();
        }.bind(this));
        this.AMQPConnection.connection.on('unblocked', function () {
            logger.debug(util.format('Connection unblocked %s to queue', process.title));
            this.AMQPConnection.blocked = false;
        }.bind(this));
        return next();
    }.bind(this));
};
MicroService.prototype.connectChannel = function (next) {
    logger.debug(util.format('Creating channel for %s', process.title));
    this.AMQPConnection.connection.createChannel(function (err, channel) {
        if (err)
            return next(err);
        this.AMQPConnection.channel = channel;
        this.AMQPConnection.channel.prefetch(1);
        this.AMQPConnection.channel.on('error', function (err) {
            if (this.AMQPConnection.connection && !this.AMQPConnection.closing) {
                logger.error(util.format('RabbitMQ connection failed with err: %s', err));
            }
        }.bind(this));
        this.AMQPConnection.channel.on('close', function () {
            logger.debug(util.format('Closed connection from %s to Q', process.title));
            if (!this.AMQPConnection.closing) {
                this.timeoutLength = 1;
                this.timeoutLimit = 180;
                return this.rabbitMQError('Unexpectedly disconnected from the message queue');
            }
        }.bind(this));
        return next();
    }.bind(this));
};
MicroService.prototype.pollBlockedStatus = function () {
    if (!this.AMQPConnection.blocked)
        return;
    logger.warn(util.format('RabbitMQ connection for %s is blocked by the server. ' +
        'Check RabbitMQ resources.', process.title));
    setTimeout(this.pollBlockedStatus.bind(this), 60 * 1000);
};
MicroService.prototype.rabbitMQError = function (err) {
    logger.error(util.inspect(err));
    logger.debug(util.format('Since an error occurred, re-connecting %s', process.title));
    async.series([
        this.disconnectQConnection.bind(this)
    ], function () {
        this.retry();
    }.bind(this));
};
MicroService.prototype.disconnectQConnection = async function (next) {
    try {
        this.AMQPConnection.closing = true;
        if (this.AMQPConnection.channel)
            await this.AMQPConnection.channel.close();
        if (this.AMQPConnection.connection)
            await this.AMQPConnection.connection.close();
    }
    catch (ex) {
        logger.warn(util.format('Failed to close connection from %s to RabbitMQ', process.title));
    }
    this.AMQPConnection.connection = null;
    this.AMQPConnection.channel = null;
    return next();
};
MicroService.prototype.retry = function () {
    this.timeoutLength *= 2;
    if (this.timeoutLength > this.timeoutLimit)
        this.timeoutLength = 1;
    logger.debug(util.format('Waiting for %s seconds before re-connecting %s', this.timeoutLength, process.title));
    setTimeout(this.establishRabbitMQConnection.bind(this), this.timeoutLength * 1000);
};
MicroService.prototype.connectToQueue = function (next) {
    var who = process.title + '|connectToQueue';
    logger.info(who, 'is connecting to the queue');
    var listener = require('../workers/signalsListener.js');
    this.AMQPConnection.channel.consume(config.queue, async function (message) {
        if (message === null) {
            return this.rabbitMQError('RabbitMQ consumer lost');
        }
        var messageContents;
        try {
            messageContents = JSON.parse(message.content.toString());
        }
        catch (exception) {
            logger.warn('Failed to parse message', util.inspect(exception));
            this.AMQPConnection.channel.nack(message, false, false);
            return;
        }
        if (messageContents.payload && messageContents.payload.isInternal) {
            global.io.of('/').adapter.remoteJoin(messageContents.payload.socketId, messageContents.payload.room).catch((err) => {
                logger.error(`Failed to remoteJoin for socketId: ${messageContents.payload.socketId}, reason: ${err}`);
            });
        }
        listener(messageContents, function (rejectMsgWithoutRequeue) {
            if (rejectMsgWithoutRequeue)
                this.AMQPConnection.channel.nack(message, false, false);
            else
                this.AMQPConnection.channel.ack(message);
        }.bind(this));
    }.bind(this), {}, function (err) {
        if (err)
            return next(err);
        logger.debug(util.format('%s is listening to Queue %s', process.title, config.queue));
        return next();
    }.bind(this));
};
MicroService.prototype.connectToRedis = function (next) {
    var who = process.title + '|connectToRedis';
    logger.info(who, 'is connecting to the redis server');
    var redisUrl = config.redisUrl;
    var parsedRedisUrl = redisUrl.split(':');
    var host = parsedRedisUrl[0];
    var port = parsedRedisUrl[1];
    var password = null;
    var sentinelPort = config.sentinelPort;
    if (parsedRedisUrl.length > 2) {
        password = parsedRedisUrl[2];
    }
    var ioServer = require('socket.io')(global.httpServer, {
        allowEIO3: true
    });
    var redisio = require('socket.io-redis');
    var Redis = require('ioredis');
    var pubClient;
    var subClient;
    var redisConDetails = {
        port: port,
        host: host
    };
    if (global.config.redisClusterEnabled === true) {
        if (!_.isUndefined(password)) {
            pubClient = new Redis.Cluster([redisConDetails], { password: password });
            subClient = new Redis.Cluster([redisConDetails], { password: password });
        }
        else {
            pubClient = new Redis.Cluster([redisConDetails]);
            subClient = new Redis.Cluster([redisConDetails]);
        }
    }
    else {
        redisConDetails = {
            name: config.sentinelGroup,
            sentinels: [
                {
                    host: host,
                    port: sentinelPort
                }
            ]
        };
        if (!_.isUndefined(password)) {
            redisConDetails['password'] = password;
            redisConDetails['sentinelPassword'] = password;
        }
        pubClient = new Redis(redisConDetails);
        subClient = new Redis(redisConDetails);
    }
    var redisIOAdapter = redisio({ pubClient: pubClient, subClient: subClient });
    ioServer.adapter(redisIOAdapter);
    ioServer.of('/').adapter.on('error', function (err) {
        logger.error(err);
    });
    ioServer.on('connection', function (socket) {
        numConnections++;
        socket.on('join-signal', function (requestObj) {
            numJoins++;
            if (requestObj.room.includes('v2/accounts'))
                numAccRoomJoins++;
            else if (requestObj.room.includes('runs/'))
                numRunJobRoomJoins++;
            else if (requestObj.room.includes('/consoles'))
                numJobConsoleRoomJoins++;
            var roomParts = requestObj.room.split('/');
            if (roomParts.length > 3) {
                logger.warn('Ignoring signal join request as room is invalid');
                return;
            }
            var url = '';
            if (requestObj.room.startsWith('v2/'))
                url = config.internalApiUrl + '/' +
                    requestObj.room.replace('v2/', '');
            else
                url = config.internalApiUrl + '/' + requestObj.room;
            if (roomParts[0] === 'projects')
                url = config.internalApiUrl + '/projects/?projectIds=' + roomParts[1];
            else if (roomParts[0] === 'identities')
                url = config.internalApiUrl + '/identities?identityIds=' +
                    roomParts[1];
            else if (roomParts[0] === 'runs' && roomParts[2] === 'jobs')
                url = config.internalApiUrl + '/runs/?runIds=' + roomParts[1];
            else if (roomParts[0] === 'jobs' && roomParts[2] === 'consoles')
                url = config.internalApiUrl + '/jobs/?jobIds=' + roomParts[1];
            var requestOpts = {
                method: 'GET',
                url: url,
                headers: {},
                timeout: 180000
            };
            if (requestObj.accessToken) {
                requestOpts.headers.Authorization = 'Bearer ' +
                    requestObj.accessToken;
                requestOpts.headers['jfrog-pipelines-session-id'] =
                    requestObj['jfrog-pipelines-session-id'];
            }
            var taskParams = {
                requestObj: requestObj,
                requestOpts: requestOpts,
                roomParts: roomParts,
                socket: socket
            };
            return doPermissionsCheck(taskParams);
        });
        socket.on('leave', function (object) {
            numLeaves++;
            socket.leave(object.room);
        });
        socket.on('disconnect', function () {
            numDisconnections++;
        });
    });
    global.io = ioServer;
    return next();
};
MicroService.prototype.initializeRoutes = function (next) {
    if (this.isRouteInitSuccess)
        return next();
    var who = process.title + '|initializeRoutes';
    logger.info(who, 'is initializing routes');
    routes((function (err) {
        if (err)
            return next('Failed to initialize routes!: ' + util.inspect(err));
        this.isRouteInitSuccess = true;
        return next();
    }).bind(this));
};
function joinSocket(socket, room) {
    socket.send('Player ' + socket.id + ' joining signal ' +
        room + ' ' + Date.now());
    socket.join(room);
}
function doPermissionsCheck(taskParams) {
    var requestOpts = taskParams.requestOpts;
    var requestObj = taskParams.requestObj;
    var roomParts = taskParams.roomParts;
    var socket = taskParams.socket;
    request(requestOpts, function (err, res, body) {
        if (!res || res.statusCode >= 299) {
            logger.error('Sockets could not access: ' + requestObj.room, body);
            return;
        }
        try {
            if (roomParts[0] === 'runs' && roomParts[2] === 'jobs' &&
                JSON.parse(body).length === 0) {
                logger.error('Sockets could not access: ' + requestObj.room +
                    ' for url: ' + requestOpts.url, body);
                return;
            }
        }
        catch (e) {
            logger.error('Invalid response received', body);
            logger.error('Sockets could not access: ' +
                requestObj.room + ' for url: ' + requestOpts.url, body);
            return;
        }
        joinSocket(socket, requestObj.room);
        return;
    });
}
//# sourceMappingURL=MicroService.js.map