'use strict';
var self = redisMonitor;
module.exports = self;
var redis = require('redis');
var util = require('util');
var _ = require('lodash');
var redisHealthCheckRetrySettings = {
    currentDelayMS: 0,
    maxDelayMS: 180 * 1000,
    defaultDelayMS: 5 * 1000
};
function redisMonitor(bag, callback) {
    var who = bag.who + '|' + redisMonitor.name;
    logger.trace(who, 'Started');
    var innerBag = {
        who: who
    };
    _doRedisHealthCheck(innerBag, function (err) {
        var who = innerBag.who + '|' + _doRedisHealthCheck.name + '|callback';
        logger.trace(who, 'Started');
        var settings = redisHealthCheckRetrySettings;
        if (err) {
            settings.currentDelayMS *= 2;
            if (settings.currentDelayMS > settings.maxDelayMS)
                settings.currentDelayMS = settings.defaultDelayMS;
            logger.warn(who, 'Redis is unhealthy', err);
        }
        else {
            settings.currentDelayMS = settings.defaultDelayMS;
            logger.trace(who, 'Redis healthy');
        }
        logger.trace(who, util.format('Checking redis health check in next %s', redisHealthCheckRetrySettings.currentDelayMS));
        setTimeout(function () {
            redisMonitor(bag);
        }, redisHealthCheckRetrySettings.currentDelayMS);
        if (callback)
            callback(err);
    });
}
function _doRedisHealthCheck(bag, callback) {
    var who = bag.who + '|' + _doRedisHealthCheck.name;
    logger.trace(who, 'Started');
    var parsedRedisUrl = config.redisUrl.split(':');
    var host = parsedRedisUrl[0];
    var port = parsedRedisUrl[1];
    var sentinelPort = config.sentinelPort;
    var password = null;
    if (parsedRedisUrl.length > 2) {
        password = parsedRedisUrl[2];
    }
    let options = {};
    if (password) {
        options = { password: password };
    }
    var client = redis.createClient(sentinelPort, host, options);
    client.on('ready', function () {
        client.end(false);
        callback();
    });
    client.on('error', function (err) {
        client.end(false);
        callback(err);
    });
}
//# sourceMappingURL=redisMonitor.js.map